import sys
import xmlrpclib
from PySide import QtGui, QtCore, QtUiTools

# Import the compiled UI module
import gui_resources


class CCurrencyThread(QtCore.QThread):

    calculated = QtCore.Signal(str)

    def __init__(self):
        super(CCurrencyThread, self).__init__()
        self.mProxy = xmlrpclib.ServerProxy("http://foxrate.org/rpc/")
        self.mCurSrc = "EUR"
        self.mCurDst = "USD"
        self.mVal = 100

    def setArgs(self, pCurSrc, pCurDst, pVal):
        self.mCurSrc = pCurSrc
        self.mCurDst = pCurDst
        self.mVal = pVal

    def run(self):
        lResult = self.mProxy.foxrate.currencyConvert(\
                self.mCurSrc, \
                self.mCurDst, \
                self.mVal)
        if lResult['message'] != '"N/A"':
            self.calculated.emit(str(lResult['amount']))
        else:
            self.calculated.emit('')


class CMain(object):
    def __init__(self, parent=None):
        super(CMain, self).__init__()

        self.mApp = QtGui.QApplication(sys.argv)
        self.mThread = CCurrencyThread()
        self.mMutex = QtCore.QMutex()

        lLoader = QtUiTools.QUiLoader()

        lFile = QtCore.QFile(":/forms/mainwindow.ui")
        lFile.open(QtCore.QFile.ReadOnly)
        self.mUi = lLoader.load(lFile)
        lFile.close()
        self.mIcon = QtGui.QIcon(":/images/icon.png")
        self.mUi.setWindowIcon(self.mIcon)

        lFile = QtCore.QFile(":/forms/customwidget.ui")
        lFile.open(QtCore.QFile.ReadOnly)
        self.customwidget = lLoader.load(lFile)
        lFile.close()
        self.mUi.setCentralWidget(self.customwidget)

        # actions
        self.mUi.actionExit.triggered.connect(self.on_actionExit_triggered)
        self.mUi.actionAbout.triggered.connect(self.on_actionAbout_triggered)
        self.customwidget.processButton.clicked.connect(\
                self.on_processButton_clicked)
        self.mThread.calculated.connect(self.on_thread_calculated,
                QtCore.Qt.QueuedConnection)

    def on_actionExit_triggered(self):
        while self.mThread.isRunning() == True:
            QtCore.QThread.msleep(100)
        self.mUi.close()

    def on_actionAbout_triggered(self):
        QtGui.QMessageBox.about(self.mUi, "XML-RPC Currency Calculator",
                "<html>&copy; Rainer Poisel und Marcus Nutzinger, 2011</html>")

    def on_processButton_clicked(self):
        if self.mMutex.tryLock() == True:
            self.mThread.setArgs(
                    self.customwidget.sourceCurrency.text(),
                self.customwidget.destinationCurrency.text(),
                int(self.customwidget.value.text()))
            self.mThread.start()

    def on_thread_calculated(self, pAmount):
        if pAmount == '':
            self.customwidget.result.setText("")
            self.customwidget.status.setText("<html><font \
                    color=\"red\">Fehler!</font></html>")
        else:
            self.customwidget.result.setText(str(pAmount))
            self.customwidget.status.setText("OK")
        self.mMutex.unlock()

    def run(self):
        self.mUi.show()
        lReturn = self.mApp.exec_()
        sys.exit(lReturn)


def main():
    lMain = CMain()
    lMain.run()

if __name__ == "__main__":
    main()
