import sys
import xmlrpclib
from PySide import QtGui, QtCore

# Import the compiled UI module
from customwidget import Ui_customWidget
from mainwindow import Ui_MainWindow


class CCurrencyThread(QtCore.QThread):

    calculated = QtCore.Signal(str)

    def __init__(self):
        super(CCurrencyThread, self).__init__()
        self.mProxy = xmlrpclib.ServerProxy("http://foxrate.org/rpc/")
        self.mCurSrc = "EUR"
        self.mCurDst = "USD"
        self.mVal = 100

    def setArgs(self, pCurSrc, pCurDst, pVal):
        self.mCurSrc = pCurSrc
        self.mCurDst = pCurDst
        self.mVal = pVal

    def run(self):
        lResult = self.mProxy.foxrate.currencyConvert(\
                self.mCurSrc, \
                self.mCurDst, \
                self.mVal)
        if lResult['message'] != '"N/A"':
            self.calculated.emit(str(lResult['amount']))
        else:
            self.calculated.emit('')


# Create a class for our main window
class Gui_Qt(QtGui.QMainWindow):

    def __init__(self, parent=None):
        super(Gui_Qt, self).__init__(parent)

        self.mThread = CCurrencyThread()
        self.mMutex = QtCore.QMutex()

        self.mUi = Ui_MainWindow()
        self.mUi.setupUi(self)

        self.centralwidget = QtGui.QWidget()
        self.customwidget = Ui_customWidget()
        self.customwidget.setupUi(self.centralwidget)
        self.setCentralWidget(self.centralwidget)

        # actions
        self.mUi.actionExit.triggered.connect(self.on_actionExit_triggered)
        self.mUi.actionAbout.triggered.connect(self.on_actionAbout_triggered)
        self.customwidget.processButton.clicked.connect(\
                self.on_processButton_clicked)
        self.mThread.calculated.connect(self.on_thread_calculated,
                QtCore.Qt.QueuedConnection)

    def on_actionExit_triggered(self):
        while self.mThread.isRunning() == True:
            QtCore.QThread.msleep(100)
        self.close()

    def on_actionAbout_triggered(self):
        QtGui.QMessageBox.about(self, "XML-RPC Currency Calculator",
                "<html>&copy; Rainer Poisel und Marcus Nutzinger, 2011</html>")

    def on_processButton_clicked(self):
        if self.mMutex.tryLock() == True:
            self.mThread.setArgs(
                    self.customwidget.sourceCurrency.text(),
                self.customwidget.destinationCurrency.text(),
                int(self.customwidget.value.text()))
            self.mThread.start()

    def on_thread_calculated(self, pAmount):
        if pAmount == '':
            self.customwidget.result.setText("")
            self.customwidget.status.setText("<html><font \
                    color=\"red\">Fehler!</font></html>")
        else:
            self.customwidget.result.setText(str(pAmount))
            self.customwidget.status.setText("OK")
        self.mMutex.unlock()


class CMain:
    def __init__(self):
        self.mApp = QtGui.QApplication(sys.argv)
        self.mWindow = Gui_Qt()

    def run(self):
        self.mWindow.show()
        lReturn = self.mApp.exec_()
        sys.exit(lReturn)


def main():
    lMain = CMain()
    lMain.run()

if __name__ == "__main__":
    main()
